import { ArgumentOutOfRangeException } from '@mobicoop/ddd-library';
import { Role } from '@modules/ad/core/domain/ad.types';
import { Target } from '@modules/ad/core/domain/candidate.types';
import { ActorTime } from '@modules/ad/core/domain/value-objects/actor-time.value-object';
import { JourneyItem } from '@modules/ad/core/domain/value-objects/journey-item.value-object';

describe('Journey item value object', () => {
  it('should create a journey item value object', () => {
    const journeyItemVO: JourneyItem = new JourneyItem({
      lat: 48.689445,
      lon: 6.17651,
      duration: 1545,
      distance: 48754,
      actorTimes: [
        new ActorTime({
          role: Role.DRIVER,
          target: Target.NEUTRAL,
          firstDatetime: new Date('2023-09-01 07:00'),
          firstMinDatetime: new Date('2023-09-01 06:45'),
          firstMaxDatetime: new Date('2023-09-01 07:15'),
          lastDatetime: new Date('2024-08-30 07:00'),
          lastMinDatetime: new Date('2024-08-30 06:45'),
          lastMaxDatetime: new Date('2024-08-30 07:15'),
        }),
      ],
    });
    expect(journeyItemVO.duration).toBe(1545);
    expect(journeyItemVO.distance).toBe(48754);
    expect(journeyItemVO.lon).toBe(6.17651);
    expect(journeyItemVO.lat).toBe(48.689445);
    expect(journeyItemVO.actorTimes[0].firstMaxDatetime.getUTCMinutes()).toBe(
      15,
    );
  });
  it('should throw an error if actorTimes is too short', () => {
    expect(
      () =>
        new JourneyItem({
          lat: 48.689445,
          lon: 6.17651,
          duration: 1545,
          distance: 48754,
          actorTimes: [],
        }),
    ).toThrow(ArgumentOutOfRangeException);
  });
});
