import {
  ArgumentInvalidException,
  ArgumentOutOfRangeException,
} from '@mobicoop/ddd-library';
import { ScheduleItem } from '@modules/ad/core/domain/value-objects/schedule-item.value-object';

describe('Schedule item value object', () => {
  it('should create a schedule item value object', () => {
    const scheduleItemVO = new ScheduleItem({
      day: 0,
      time: '07:00',
      margin: 900,
    });
    expect(scheduleItemVO.day).toBe(0);
    expect(scheduleItemVO.time).toBe('07:00');
    expect(scheduleItemVO.margin).toBe(900);
  });
  it('should throw an exception if day is invalid', () => {
    expect(() => {
      new ScheduleItem({
        day: 7,
        time: '07:00',
        margin: 900,
      });
    }).toThrow(ArgumentOutOfRangeException);
  });
  it('should throw an exception if time is invalid', () => {
    expect(() => {
      new ScheduleItem({
        day: 0,
        time: '07,00',
        margin: 900,
      });
    }).toThrow(ArgumentInvalidException);
  });
  it('should throw an exception if the hour of the time is invalid', () => {
    expect(() => {
      new ScheduleItem({
        day: 0,
        time: '25:00',
        margin: 900,
      });
    }).toThrow(ArgumentInvalidException);
  });
  it('should throw an exception if the minutes of the time are invalid', () => {
    expect(() => {
      new ScheduleItem({
        day: 0,
        time: '07:63',
        margin: 900,
      });
    }).toThrow(ArgumentInvalidException);
  });
});
