import { Frequency, Role } from '@modules/ad/core/domain/ad.types';
import { Target } from '@modules/ad/core/domain/candidate.types';
import { MatchEntity } from '@modules/ad/core/domain/match.entity';
import { MatchingEntity } from '@modules/ad/core/domain/matching.entity';
import { MatchingMapper } from '@modules/ad/matching.mapper';
import { Test } from '@nestjs/testing';

describe('Matching Mapper', () => {
  let matchingMapper: MatchingMapper;

  beforeAll(async () => {
    const module = await Test.createTestingModule({
      providers: [MatchingMapper],
    }).compile();
    matchingMapper = module.get<MatchingMapper>(MatchingMapper);
  });

  it('should be defined', () => {
    expect(matchingMapper).toBeDefined();
  });

  it('should map domain entity to persistence', async () => {
    const matchingEntity: MatchingEntity = new MatchingEntity({
      id: '644a7cb3-6436-4db5-850d-b4c7421d4b97',
      createdAt: new Date('2023-08-20T09:48:00Z'),
      updatedAt: new Date('2023-08-20T09:48:00Z'),
      props: {
        matches: [
          new MatchEntity({
            id: '4bd4e90b-ffba-4f5f-b904-48ad0667a1d7',
            createdAt: new Date('2023-08-30T08:45:00Z'),
            updatedAt: new Date('2023-08-30T08:45:00Z'),
            props: {
              adId: 'dd937edf-1264-4868-b073-d1952abe30b1',
              role: Role.DRIVER,
              frequency: Frequency.PUNCTUAL,
              distance: 356041,
              duration: 12647,
              initialDistance: 348745,
              initialDuration: 12105,
              distanceDetour: 7296,
              durationDetour: 542,
              distanceDetourPercentage: 4.1,
              durationDetourPercentage: 3.8,
              journeys: [
                {
                  firstDate: new Date('2023-09-01'),
                  lastDate: new Date('2023-09-01'),
                  journeyItems: [
                    {
                      lon: 6.389745,
                      lat: 48.32644,
                      duration: 0,
                      distance: 0,
                      actorTimes: [
                        {
                          role: Role.DRIVER,
                          target: Target.START,
                          firstDatetime: new Date('2023-09-01T07:00:00Z'),
                          firstMinDatetime: new Date('2023-09-01T06:45:00Z'),
                          firstMaxDatetime: new Date('2023-09-01T07:15:00Z'),
                          lastDatetime: new Date('2023-09-01T07:00:00Z'),
                          lastMinDatetime: new Date('2023-09-01T06:45:00Z'),
                          lastMaxDatetime: new Date('2023-09-01T07:15:00Z'),
                        },
                        {
                          role: Role.PASSENGER,
                          target: Target.START,
                          firstDatetime: new Date('2023-09-01T07:00:00Z'),
                          firstMinDatetime: new Date('2023-09-01T06:45:00Z'),
                          firstMaxDatetime: new Date('2023-09-01T07:15:00Z'),
                          lastDatetime: new Date('2023-09-01T07:00:00Z'),
                          lastMinDatetime: new Date('2023-09-01T06:45:00Z'),
                          lastMaxDatetime: new Date('2023-09-01T07:15:00Z'),
                        },
                      ],
                    },
                    {
                      lon: 6.984567,
                      lat: 48.021548,
                      distance: 356041,
                      duration: 12647,
                      actorTimes: [
                        {
                          role: Role.DRIVER,
                          target: Target.FINISH,
                          firstDatetime: new Date('2023-09-01T07:00:00Z'),
                          firstMinDatetime: new Date('2023-09-01T06:45:00Z'),
                          firstMaxDatetime: new Date('2023-09-01T07:15:00Z'),
                          lastDatetime: new Date('2023-09-01T07:00:00Z'),
                          lastMinDatetime: new Date('2023-09-01T06:45:00Z'),
                          lastMaxDatetime: new Date('2023-09-01T07:15:00Z'),
                        },
                        {
                          role: Role.PASSENGER,
                          target: Target.FINISH,
                          firstDatetime: new Date('2023-09-01T07:00:00Z'),
                          firstMinDatetime: new Date('2023-09-01T06:45:00Z'),
                          firstMaxDatetime: new Date('2023-09-01T07:15:00Z'),
                          lastDatetime: new Date('2023-09-01T07:00:00Z'),
                          lastMinDatetime: new Date('2023-09-01T06:45:00Z'),
                          lastMaxDatetime: new Date('2023-09-01T07:15:00Z'),
                        },
                      ],
                    },
                  ],
                },
              ],
            },
          }),
        ],
        query: {
          driver: false,
          passenger: true,
          frequency: Frequency.PUNCTUAL,
          fromDate: '2023-09-01',
          toDate: '2023-09-01',
          schedule: [
            {
              day: 5,
              time: '06:40',
              margin: 900,
            },
          ],
          seatsProposed: 3,
          seatsRequested: 1,
          strict: true,
          waypoints: [
            {
              lon: 6.389745,
              lat: 48.32644,
            },
            {
              lon: 6.984567,
              lat: 48.021548,
            },
          ],
          algorithmType: 'PASSENGER_ORIENTED',
          remoteness: 15000,
          useProportion: true,
          proportion: 0.3,
          useAzimuth: true,
          azimuthMargin: 10,
          maxDetourDistanceRatio: 0.3,
          maxDetourDurationRatio: 0.3,
        },
      },
    });
    const mapped: string = matchingMapper.toPersistence(matchingEntity);
    expect(mapped).toBe(
      '{"id":"644a7cb3-6436-4db5-850d-b4c7421d4b97","createdAt":"2023-08-20T09:48:00.000Z","updatedAt":"2023-08-20T09:48:00.000Z","matches":[{"adId":"dd937edf-1264-4868-b073-d1952abe30b1","role":"DRIVER","frequency":"PUNCTUAL","distance":356041,"duration":12647,"initialDistance":348745,"initialDuration":12105,"distanceDetour":7296,"durationDetour":542,"distanceDetourPercentage":4.1,"durationDetourPercentage":3.8,"journeys":[{"firstDate":"2023-09-01T00:00:00.000Z","lastDate":"2023-09-01T00:00:00.000Z","journeyItems":[{"lon":6.389745,"lat":48.32644,"duration":0,"distance":0,"actorTimes":[{"role":"DRIVER","target":"START","firstDatetime":"2023-09-01T07:00:00.000Z","firstMinDatetime":"2023-09-01T06:45:00.000Z","firstMaxDatetime":"2023-09-01T07:15:00.000Z","lastDatetime":"2023-09-01T07:00:00.000Z","lastMinDatetime":"2023-09-01T06:45:00.000Z","lastMaxDatetime":"2023-09-01T07:15:00.000Z"},{"role":"PASSENGER","target":"START","firstDatetime":"2023-09-01T07:00:00.000Z","firstMinDatetime":"2023-09-01T06:45:00.000Z","firstMaxDatetime":"2023-09-01T07:15:00.000Z","lastDatetime":"2023-09-01T07:00:00.000Z","lastMinDatetime":"2023-09-01T06:45:00.000Z","lastMaxDatetime":"2023-09-01T07:15:00.000Z"}]},{"lon":6.984567,"lat":48.021548,"duration":12647,"distance":356041,"actorTimes":[{"role":"DRIVER","target":"FINISH","firstDatetime":"2023-09-01T07:00:00.000Z","firstMinDatetime":"2023-09-01T06:45:00.000Z","firstMaxDatetime":"2023-09-01T07:15:00.000Z","lastDatetime":"2023-09-01T07:00:00.000Z","lastMinDatetime":"2023-09-01T06:45:00.000Z","lastMaxDatetime":"2023-09-01T07:15:00.000Z"},{"role":"PASSENGER","target":"FINISH","firstDatetime":"2023-09-01T07:00:00.000Z","firstMinDatetime":"2023-09-01T06:45:00.000Z","firstMaxDatetime":"2023-09-01T07:15:00.000Z","lastDatetime":"2023-09-01T07:00:00.000Z","lastMinDatetime":"2023-09-01T06:45:00.000Z","lastMaxDatetime":"2023-09-01T07:15:00.000Z"}]}]}]}],"query":{"driver":false,"passenger":true,"frequency":"PUNCTUAL","fromDate":"2023-09-01","toDate":"2023-09-01","schedule":[{"day":5,"time":"06:40","margin":900}],"seatsProposed":3,"seatsRequested":1,"strict":true,"waypoints":[{"lon":6.389745,"lat":48.32644},{"lon":6.984567,"lat":48.021548}],"algorithmType":"PASSENGER_ORIENTED","remoteness":15000,"useProportion":true,"proportion":0.3,"useAzimuth":true,"azimuthMargin":10,"maxDetourDistanceRatio":0.3,"maxDetourDurationRatio":0.3}}',
    );
  });

  it('should map persisted string to domain entity', async () => {
    const matchingEntity: MatchingEntity = matchingMapper.toDomain(
      '{"id":"644a7cb3-6436-4db5-850d-b4c7421d4b97","createdAt":"2023-08-20T09:48:00.000Z","updatedAt":"2023-08-20T09:48:00.000Z","matches":[{"adId":"dd937edf-1264-4868-b073-d1952abe30b1","role":"DRIVER","frequency":"PUNCTUAL","distance":356041,"duration":12647,"initialDistance":348745,"initialDuration":12105,"distanceDetour":7296,"durationDetour":542,"distanceDetourPercentage":4.1,"durationDetourPercentage":3.8,"journeys":[{"firstDate":"2023-09-01T00:00:00.000Z","lastDate":"2023-09-01T00:00:00.000Z","journeyItems":[{"lon":6.389745,"lat":48.32644,"duration":0,"distance":0,"actorTimes":[{"role":"DRIVER","target":"START","firstDatetime":"2023-09-01T07:00:00.000Z","firstMinDatetime":"2023-09-01T06:45:00.000Z","firstMaxDatetime":"2023-09-01T07:15:00.000Z","lastDatetime":"2023-09-01T07:00:00.000Z","lastMinDatetime":"2023-09-01T06:45:00.000Z","lastMaxDatetime":"2023-09-01T07:15:00.000Z"},{"role":"PASSENGER","target":"START","firstDatetime":"2023-09-01T07:00:00.000Z","firstMinDatetime":"2023-09-01T06:45:00.000Z","firstMaxDatetime":"2023-09-01T07:15:00.000Z","lastDatetime":"2023-09-01T07:00:00.000Z","lastMinDatetime":"2023-09-01T06:45:00.000Z","lastMaxDatetime":"2023-09-01T07:15:00.000Z"}]},{"lon":6.984567,"lat":48.021548,"duration":12647,"distance":356041,"actorTimes":[{"role":"DRIVER","target":"FINISH","firstDatetime":"2023-09-01T07:00:00.000Z","firstMinDatetime":"2023-09-01T06:45:00.000Z","firstMaxDatetime":"2023-09-01T07:15:00.000Z","lastDatetime":"2023-09-01T07:00:00.000Z","lastMinDatetime":"2023-09-01T06:45:00.000Z","lastMaxDatetime":"2023-09-01T07:15:00.000Z"},{"role":"PASSENGER","target":"FINISH","firstDatetime":"2023-09-01T07:00:00.000Z","firstMinDatetime":"2023-09-01T06:45:00.000Z","firstMaxDatetime":"2023-09-01T07:15:00.000Z","lastDatetime":"2023-09-01T07:00:00.000Z","lastMinDatetime":"2023-09-01T06:45:00.000Z","lastMaxDatetime":"2023-09-01T07:15:00.000Z"}]}]}]}],"query":{"driver":false,"passenger":true,"frequency":"PUNCTUAL","fromDate":"2023-09-01","toDate":"2023-09-01","schedule":[{"day":5,"time":"06:40","margin":900}],"seatsProposed":3,"seatsRequested":1,"strict":true,"waypoints":[{"lon":6.389745,"lat":48.32644},{"lon":6.984567,"lat":48.021548}],"algorithmType":"PASSENGER_ORIENTED","remoteness":15000,"useProportion":true,"proportion":0.3,"useAzimuth":true,"azimuthMargin":10,"maxDetourDistanceRatio":0.3,"maxDetourDurationRatio":0.3}}',
    );
    expect(matchingEntity.getProps().query.fromDate).toBe('2023-09-01');
    expect(matchingEntity.getProps().matches[0].getProps().adId).toBe(
      'dd937edf-1264-4868-b073-d1952abe30b1',
    );
  });
});
